/*
 * (c) 2005 by Simon Gander <simongander@gmx.ch>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package guettingen;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

class Signalhebelsperre extends Hebelsperre implements MouseListener {
	public Signalhebelsperre(boolean as, Block bk, Signal s) {
		// Int typ, Block blk
		super();
		if (as) bild = asig_k;
		else bild = esig_k;
		
		asig = as;
		block = bk;
		
		addMouseListener (this);
		gesperrt = true;
		
		sig = s;
	}
	
	public boolean umlegenOk(boolean position) {
/*		if (asig) {
			if (!block.habe_freie_bahn) lampe_ein = false;
		}
*/		return sperre_frei;
	}
	
	public void umlegen(boolean position) {
		setFreigabe(false);
		pos = position;
		if (asig) {
			if (position) {
				block.vorblocken();
			} else {
				block.blocken();
			}
		}
		if (!asig) {
			if (!position) {
				eingefahren = false;
				block.rueckmeldenMoeglich();
			}
		}
		
		drawLamps (getGraphics());
	}

	public void zugEingefahren() {
		if (!block.habe_freie_bahn) {
			block.zugEingefahren();
			if (pos) {
				eingefahren = true;
				drawLamps (getGraphics());
				
			}
		}
	}
	

	public void paint (Graphics g) {
		if (bild == null) return;
		g.drawImage (bild, 0, 0, this);
		drawLamps(g);
	}

	public void drawLampe (Graphics g, int y, Color c) {
		g.setColor(c);
		g.drawLine (28, y - 4, 31, y - 4);
		g.drawLine (27, y - 3, 32, y - 3);
		g.drawLine (26, y - 2, 33, y - 2);
		g.drawLine (26, y - 1, 33, y - 1);
		g.drawLine (26, y    , 33, y    );
		g.drawLine (26, y + 1, 33, y + 1);
		g.drawLine (27, y + 2, 32, y + 2);
		g.drawLine (28, y + 3, 31, y + 3);
	}
	
	public void drawLamps (Graphics g) {
		if (sperre_frei) drawLampe (g, 26, Color.GREEN);
		else drawLampe (g, 26, Color.DARK_GRAY);
		if (!asig) {
			if (eingefahren) drawLampe (g, 52, Color.YELLOW);
			else drawLampe (g, 52, Color.DARK_GRAY);
		}
	}
	
	public void sperren () {
		gesperrt = true;
		if (sperre_frei) {
			setFreigabe(false);
//			repaint();
			drawLamps (getGraphics());
		}
	}
	
	public void freigeben () {
		gesperrt = false;
	}
	
	public void signalHalt () {
		if (sig != null) {
			sig.Halt();
		}
		if (asig && pos) {
			setFreigabe(true);
			drawLamps (getGraphics());
		}
	}
	
	public void betriebsAufloesung () {
		if (!pos) return;
		if (asig) setFreigabe(true);
		drawLamps (getGraphics());
	}
	
	public void blocken() {
		if (asig) {
			if (sig.stellung != 0) block.blocken();
		}
	}
	
	public void mouseClicked (MouseEvent e) {
		int x, y;
		x = e.getX();
		y = e.getY();
		
		if (x < 30) {
			// Freigabe gedrueckt
			if (gesperrt) return;
			if (pos) return;
			if (asig) {
				if (!block.habe_freie_bahn) return;
				if (block.status != 0) return;
			} else {
				if (block.habe_freie_bahn) return;
//				if (block.status == 0) return;
//				if (block.rueck_moeglich) return;
			}
			setFreigabe(true);
			drawLamps (getGraphics());
		} else {
			if (y < 20) {		// Blockumgehung
				if (gesperrt) return;
				if (!asig) return;
				if (pos) return;
				setFreigabe(true);
				drawLamps (getGraphics());
			} else if (y < 40) {	// Isolierumgehung
				// Vorlufig = Freigabe, weil keine Isolierpruefung
				if (gesperrt) return;
				if (pos) return;
				if (asig) {
					if (!block.habe_freie_bahn) return;
					if (block.status != 0) return;
				} else {
					if (block.habe_freie_bahn) return;
//					if (block.status == 0) return;
//					if (block.rueck_moeglich) return;
				}
			}
		}
	}
	
	void setFreigabe(boolean frei) {
		boolean alt_frei = sperre_frei;
		sperre_frei = frei;
		if (frei != alt_frei) freigabe_sound.play();
	}
	
	public void mousePressed (MouseEvent e) { }
	public void mouseReleased (MouseEvent e) { }
	public void mouseEntered (MouseEvent e) { }
	public void mouseExited (MouseEvent e) { }
	
	boolean sperre_frei;
	boolean eingefahren;
	boolean gesperrt;
	boolean pos;
	boolean asig;
	
	Signal sig;
	Block block;
	
	static final long serialVersionUID = -1907975505493600834L;
}
